/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */

#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <stdio.h>
#include <setjmp.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>
#include <stdarg.h>
#include <syslog.h>
#include <strings.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <sys/time.h>

/* force debug mode */
#define DEBUG

#include "common.h"

#ifndef __JNIX_H
#define __JNIX_H


/*
** Connection Stuff
*/

typedef void (*jnixEvent)(void);

typedef struct jnixc_struct
{
	pool p;
	int flags;
	int state;
	int fd;
	jid user;
	char *pass;
	/* xmlstream */
	XML_Parser xparser;
	xmlnode xnode;
	int xflag;
} *jnixc, _jnixc;

/* MODE: simple connection proxy (no funky jnix stuff) */
jnixc jnix_connect(char *user, char *pass, int flags);
void jnix_disconnect(jnixc j);
/* polls this connection, callbacks are fired for incoming stuff */
void jnix_poll(jnixc j);
/* returns fd for external selects to watch and call jnix_poll on */
int jnix_fd(jnixc j);
/* return 1 if connection is online, 0 otherwise */
int jnix_isOnline(jnixc j);
/* sets callback handlers */
void jnix_setcb(int cb, jnixEvent func);

#define JNIXC_DIRECT 1	/* direct connection to server */
#define JNIXC_FASTIO 2	/* use non-blocking io */
#define JNIXC_NEWUSER 4	/* register this user */

#define JNIX_STATE_OFFLINE 0	/* these are for internal use */
#define JNIX_STATE_CONNECTING 1
#define JNIX_STATE_STREAMING 2
#define JNIX_STATE_ONLINE 3

#define JNIXCB_ONLINE 0	/* socket connected and stream started */
#define JNIXCB_OFFLINE 1	/* connection closed */
#define JNIXCB_MESSAGE 2	/* got message */
#define JNIXCB_PRESENCE 3	/* got presence information */
#define JNIXCB_IQ 4	/* got iq information */
#define JNIXCB_ROSTER 5	/* got roster information */
#define JNIXCB_AUTH 6	/* got auth/register information */

#define JNIX_OFF_BYE 1	/* server closed connection */
#define JNIX_OFF_XML 2	/* received bad xml code from server */
#define JNIX_OFF_SOCK 3	/* socket closed */




/*
** Node Stuff
*/

#define JNIXNSTRUCT pool p; int node; char *id; jid from; jid to; int type;

union jnixn_union
{
	struct jnixn_struct
	{
		JNIXNSTRUCT
	} jn;
	struct jnixm_struct
	{
		JNIXNSTRUCT
		int priority;
		char *thread;
		char *subject;
		char *body;
	} jm;
	struct jnixp_struct
	{
		JNIXNSTRUCT
		int priority;
		char *status;
	} jp;
	struct jnixq_struct
	{
		JNIXNSTRUCT
		char *xmlns;
		struct jnixitem_struct *items;
	} jq;
};

typedef union jnixn_union *jnixn;
typedef struct jnixm_struct *jnixm, _jnixm;
typedef struct jnixp_struct *jnixp, _jnixp;
typedef struct jnixq_struct *jnixq, _jnixq;

jnixn jnix_newNode(int type, ...);
void jnix_editNode(jnixn node, ...);
/*jnixn jnix_replyNode(jnixn node, ...);*/
jnixn jnix_parseNode(xmlnode xml);
void jnix_sendNode(jnixc j, jnixn node);
void jnix_freeNode(jnixn node);

/* nodes */
#define JNIXN_MSG 0
#define JNIXN_PRS 1
#define JNIXN_IQ 2

/* common args and their synonyms */
#define JNIXN_DONE 0
#define JNIXM_DONE 0
#define JNIXP_DONE 0
#define JNIXQ_DONE 0
#define JNIXN_ID 1
#define JNIXM_ID 1
#define JNIXP_ID 1
#define JNIXQ_ID 1
#define JNIXN_FROM 2
#define JNIXM_FROM 2
#define JNIXP_FROM 2
#define JNIXQ_FROM 2
#define JNIXN_TO 3
#define JNIXM_TO 3
#define JNIXP_TO 3
#define JNIXQ_TO 3
#define JNIXN_TYPE 4
#define JNIXM_TYPE 4
#define JNIXP_TYPE 4
#define JNIXQ_TYPE 4

/* message args */
#define JNIXM_BODY 21
#define JNIXM_SUBJECT 22
#define JNIXM_THREAD 23
#define JNIXM_PRIORITY 24

/* message types */
#define JNIXM_TYPE_NORMAL 0	/* default */
#define JNIXM_TYPE_ERROR 1
#define JNIXM_TYPE_CHAT 2
#define JNIXM_TYPE_GROUPCHAT 3

/* presence args */
#define JNIXP_STATUS 41
#define JNIXP_PRIORITY 42

/* presence types */
#define JNIXP_TYPE_AVAILABLE 0	/* default */
#define JNIXP_TYPE_UNAVAILABLE 1
#define JNIXP_TYPE_SUBSCRIBE 2
#define JNIXP_TYPE_SUBSCRIBED 3
#define JNIXP_TYPE_UNSUBSCRIBE 4
#define JNIXP_TYPE_UNSUBSCRIBED 5
#define JNIXP_TYPE_PROBE 6

/* query args */
#define JNIXQ_XMLNS 61

/* query types */
#define JNIXQ_TYPE_GET 0	/* default */
#define JNIXQ_TYPE_SET 1
#define JNIXQ_TYPE_RESULT 2

/* query item stuff */
typedef struct jnixitem_struct
{
	struct jnixitem_struct *next;
	struct jnixitem_struct *sub;
	struct jnixitem_struct *attr;
	char *element;
	char *value;
} *jnixitem, _jnixitem;

jnixitem jnix_addItem(jnixn node, jnixitem item, char *element, char *value, ...);
jnixitem jnix_findItem(jnixitem item, char *element);
char *_jnix_item2str(pool p, jnixitem item);	/* private function */
void _jnix_parseItem(pool p, jnixitem *list, xmlnode x);	/* private function */




/*
** Roster Stuff
*/

typedef struct jnixr_struct
{
	pool p;
	struct jnixritem_struct *roster;
} *jnixr, _jnixr;

typedef struct jnixritem_struct
{
	struct jnixritem_struct *next;
	int type;
	char *group;
	jid user;
	char *comment;
} *jnixritem, _jnixritem;

#define JNIXR_TYPE_USER 0
#define JNIXR_TYPE_GROUP 1

#define JNIX_ROSTER_ADD 0
#define JNIX_ROSTER_REMOVE 1



/* roster stuff */
void jnix_getRoster(jnixc c);
void jnix_getRosterGroup(jnixc c, char *group);
void jnix_setRoster(jnixc c, int cmd, char *group, char *user, char *comment);
void jnix_freeRoster(jnixr rost);

jnixr _jnix_parseRoster(xmlnode x);	/* private func */
void _jnix_addRosterItem(jnixr rost, int type, char *group, char *user, char *comment); /* private func */



/*
** Auth & Register Stuff
*/

typedef struct jnixa_struct
{
	pool p;
	int type;
	jid user;
	char *msg;
} *jnixa, _jnixa;

#define JNIXA_OK 0
#define JNIXA_FAIL 1
#define JNIXA_REGOK 2
#define JNIXA_REGFAIL 3


void jnix_auth(jnixc j, char *user, char *pass);
void jnix_register(jnixc j, char *username, char *pass);
void jnix_freeAuth(jnixa a);
jnixa _jnix_parseAuth(int f, jnixn node);	/* private func */



#endif	/* __JNIX_H */
