/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */

#include "jnix.h"
#include "xmlstream.h"

jnixitem _jnix_addItem(pool p, jnixitem *list, char *element, char *value);


jnixitem jnix_addItem(jnixn node, jnixitem item, char *element, char *value, ...)
{
	jnixitem tmp;
	va_list ap;
	char *arg1,*arg2;
	pool p = node->jn.p;

	if(item) {
		tmp = jnix_findItem(item->sub,element);
		if(!tmp) tmp = _jnix_addItem(p,&item->sub,element,value);
	} else {
		tmp = jnix_findItem(node->jq.items,element);
		if(!tmp) tmp = _jnix_addItem(p,&node->jq.items,element,value);
	}

	va_start(ap,value);
	while(1) {
		arg1 = va_arg(ap,char*);
		if(arg1==NULL) return(tmp);
		arg2 = va_arg(ap,char*);
		_jnix_addItem(p,&tmp->attr,arg1,arg2);
	}
}


jnixitem jnix_findItem(jnixitem item, char *element)
{
	jnixitem tmp=item;

	while(tmp) {
		if(j_strcmp(tmp->element,element)==0) return(tmp);
		tmp = tmp->next;
	}
	return(NULL);
}


/*
** Private Functions
*/

void _jnix_subitem2str(spool xml, jnixitem item)
{
	jnixitem tmp=item,tmp2;

	while(tmp) {
		spooler(xml,"<",tmp->element,xml);
		tmp2 = tmp->attr;
		while(tmp2) {
			spooler(xml," ",tmp2->element,"='",tmp2->value,"'",xml);
			tmp2 = tmp2->next;
		}
		if(tmp->sub) {
			spool_add(xml,">");
			_jnix_subitem2str(xml,tmp->sub);
			spooler(xml,"</",tmp->element,">",xml);
		} else {
			if(tmp->value)
				spooler(xml,">",tmp->value,"</",tmp->element,">",xml);
			else
				spool_add(xml,"/>");
		}
		tmp = tmp->next;
	}
}

char *_jnix_item2str(pool p, jnixitem item)
{
	char *buf;
	spool xml;

	xml = spool_new(p);
	_jnix_subitem2str(xml,item);
	buf = spool_print(xml);
	return(buf);
}


void _jnix_parseItem(pool p, jnixitem *list, xmlnode x)
{
	xmlnode tmp=x,tmp2,tmp3;
	jnixitem item;

	while(tmp) {
		tmp3 = xmlnode_get_firstchild(tmp);
		item = _jnix_addItem(p,list,xmlnode_get_name(tmp),xmlnode_get_data(tmp3));
		tmp2 = xmlnode_get_firstattrib(tmp);
		while(tmp2) {
			_jnix_addItem(p,&item->attr,xmlnode_get_name(tmp2),xmlnode_get_data(tmp2));
			tmp2 = xmlnode_get_nextsibling(tmp2);
		}
		while(tmp3) {
			if(xmlnode_get_type(tmp3)==NTYPE_TAG)
				_jnix_parseItem(p,&item->sub,tmp3);
			tmp3 = xmlnode_get_nextsibling(tmp3);
		}
		tmp = xmlnode_get_nextsibling(tmp);
	}
}


jnixitem _jnix_addItem(pool p, jnixitem *list, char *element, char *value)
{
	jnixitem tmp;

	tmp = (jnixitem)pmalloc(p,sizeof(_jnixitem));
	memset(tmp,0,sizeof(_jnixitem));
	tmp->element = pstrdup(p,element);
	if(value) tmp->value = pstrdup(p,value);
	if(*list) tmp->next = *list;
	*list = tmp;
	return(tmp);
}
