/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */

#include "jnix.h"
#include "xmlstream.h"

typedef void (*_jnixcbe_on)(jnixc j);
typedef void (*_jnixcbe_off)(jnixc j, int err);
typedef void (*_jnixcbe_msg)(jnixc j, jnixm msg);
typedef void (*_jnixcbe_prs)(jnixc j, jnixp prs);
typedef void (*_jnixcbe_iq)(jnixc j, jnixq iq);
typedef void (*_jnixcbe_auth)(jnixc j, jnixa auth);
typedef void (*_jnixcbe_roster)(jnixc j, jnixr roster);

_jnixcbe_on _jnixcb_on=NULL;
_jnixcbe_off _jnixcb_off=NULL;
_jnixcbe_msg _jnixcb_msg=NULL;
_jnixcbe_prs _jnixcb_prs=NULL;
_jnixcbe_iq _jnixcb_iq=NULL;
_jnixcbe_auth _jnixcb_auth=NULL;
_jnixcbe_roster _jnixcb_roster=NULL;

void _jnix_parsenode(jnixc j, xmlnode x, int type);


jnixc jnix_connect(char *user, char *pass, int flags)
{
	jnixc j;
	pool p;

	p = pool_new();
	if (NULL == p) return(NULL);
	j = (jnixc)pmalloc(p,sizeof(_jnixc));
	if (NULL == j) return(NULL);
	memset(j,0,sizeof(_jnixc));
	j->p = p;

	j->user = jid_new(p,user);
	j->pass = pstrdup(p,pass);
	j->flags = flags;

	j->state = JNIX_STATE_OFFLINE;

	if(j->flags&JNIXC_FASTIO) {
		/* good place to add nonblocking connect */
		return(NULL);
	} else {
		j->fd = make_netsocket(5222,j->user->server,NETSOCKET_CLIENT);
		if (j->fd < 0) return(NULL);
		xmlstream_start(j,xmlstream_header("jabber:client",j->user->server,NULL));
		j->state = JNIX_STATE_STREAMING;
	}

	return(j);
}


void jnix_disconnect(jnixc j)
{
	xmlstream_stop(j,1,NULL);
	pool_free(j->p);
}


int jnix_fd(jnixc j)
{
	return(j->fd);
}


int jnix_isOnline(jnixc j)
{
	if (JNIX_STATE_ONLINE != j->state) return(0);
	return(1);
}


void jnix_setcb(int cb, jnixEvent func)
{
	switch(cb) {
		case JNIXCB_ONLINE: _jnixcb_on=(_jnixcbe_on)func; break;
		case JNIXCB_MESSAGE: _jnixcb_msg=(_jnixcbe_msg)func; break;
		case JNIXCB_PRESENCE: _jnixcb_prs=(_jnixcbe_prs)func; break;
		case JNIXCB_IQ: _jnixcb_iq=(_jnixcbe_iq)func; break;
		case JNIXCB_ROSTER: _jnixcb_roster=(_jnixcbe_roster)func; break;
		case JNIXCB_AUTH: _jnixcb_auth=(_jnixcbe_auth)func; break;
		case JNIXCB_OFFLINE: _jnixcb_off=(_jnixcbe_off)func; break;
	}
}


void jnix_poll(jnixc j)
{
	fd_set fds;
	struct timeval tv;
	int s;

	if (!j) return;
	if (JNIX_STATE_OFFLINE == j->state) return;

	s = jnix_fd(j);
	FD_ZERO(&fds);
	FD_SET(s,&fds);
	tv.tv_sec = 0;
	tv.tv_usec = 25;
	if (select(s+1,&fds,NULL,NULL,&tv) <= 0) return;

	xmlstream_read(j);
}


/*
** private functions
*/

void _jnix_parsenode(jnixc j, xmlnode x, int type)
{
	jnixn node;

	switch(type) {
		case XMLSTREAM_ROOT:
			j->state = JNIX_STATE_ONLINE;
			if (j->flags&JNIXC_NEWUSER)
				jnix_register(j, j->user->user, j->pass);
			else
				jnix_auth(j, jid_full(j->user), j->pass);
			if(_jnixcb_on) (_jnixcb_on)(j);
			break;
		case XMLSTREAM_NODE:
			node = jnix_parseNode(x);
			if(!node) break;
			if(!node->jn.to) node->jn.to = j->user;
			if (node->jn.node==JNIXN_MSG) {
					if (_jnixcb_msg) (_jnixcb_msg)(j,(jnixm)node);
			} else if (node->jn.node==JNIXN_PRS) {
					if (_jnixcb_prs) (_jnixcb_prs)(j,(jnixp)node);
			} else if (node->jn.node==JNIXN_IQ) {
					if (j->flags&JNIXC_NEWUSER && j_strcmp(node->jq.xmlns,"jabber:iq:register")==0) {
						jnix_auth(j, jid_full(j->user), j->pass);
						j->flags = j->flags ^ JNIXC_NEWUSER;
					}
					if (_jnixcb_roster && j_strcmp(node->jq.xmlns, "jabber:iq:roster")==0) {
						jnixr rost;
						rost = _jnix_parseRoster(x);
						(_jnixcb_roster)(j,rost);
					} else if (_jnixcb_auth && j_strcmp(node->jq.xmlns,"jabber:iq:auth")==0) {
						jnixa auth;
						auth = _jnix_parseAuth(0,node);
						if(auth) (_jnixcb_auth)(j,auth);
					} else if (_jnixcb_auth && j_strcmp(node->jq.xmlns,"jabber:iq:register")==0) {
						jnixa auth;
						auth = _jnix_parseAuth(1,node);
						if(auth) (_jnixcb_auth)(j,auth);
					} else if (_jnixcb_iq) {
						(_jnixcb_iq)(j,(jnixq)node);
					}
			}
			break;
		case XMLSTREAM_CLOSE:
			xmlstream_stop(j,0,NULL);
			if (_jnixcb_off) (_jnixcb_off)(j,JNIX_OFF_BYE);
			break;
		case XMLSTREAM_XMLERR:
			xmlstream_stop(j,1,"Bad XML");
			if (_jnixcb_off) (_jnixcb_off)(j,JNIX_OFF_XML);
			break;
		case XMLSTREAM_SOCKERR:
			xmlstream_stop(j,0,NULL);
			if (_jnixcb_off) (_jnixcb_off)(j,JNIX_OFF_SOCK);
			break;
	}
}
