/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */

#include "jnix.h"
#include "xmlstream.h"

struct _jnix_typelist_str
{
	char *name;
	int node;
	int type;
};

struct _jnix_typelist_str _jnix_typelist[] = {
	{ "normal", JNIXN_MSG, JNIXM_TYPE_NORMAL },
	{ "error", JNIXN_MSG, JNIXM_TYPE_ERROR },
	{ "chat", JNIXN_MSG, JNIXM_TYPE_CHAT },
	{ "groupchat", JNIXN_MSG, JNIXM_TYPE_GROUPCHAT },
	{ "available", JNIXN_PRS, JNIXP_TYPE_AVAILABLE },
	{ "unavailable", JNIXN_PRS, JNIXP_TYPE_UNAVAILABLE },
	{ "subscribe", JNIXN_PRS, JNIXP_TYPE_SUBSCRIBE },
	{ "subscribed", JNIXN_PRS, JNIXP_TYPE_SUBSCRIBED },
	{ "unsubscribe", JNIXN_PRS, JNIXP_TYPE_UNSUBSCRIBE },
	{ "unsubscribed", JNIXN_PRS, JNIXP_TYPE_UNSUBSCRIBED },
	{ "probe", JNIXN_PRS, JNIXP_TYPE_PROBE },
	{ "get", JNIXN_IQ, JNIXQ_TYPE_GET },
	{ "set", JNIXN_IQ, JNIXQ_TYPE_SET },
	{ "result", JNIXN_IQ, JNIXQ_TYPE_RESULT },
	{ NULL, 0, 0 }
};

void _jnix_setNode(jnixn node, pool p, va_list ap);


jnixn jnix_newNode(int type, ...)
{
	jnixn node;
	pool p;
	int sz;
	va_list ap;

	p = pool_new();
	switch(type) {
		case JNIXN_MSG: sz=sizeof(struct jnixm_struct); break;
		case JNIXN_PRS: sz=sizeof(struct jnixp_struct); break;
		case JNIXN_IQ: sz=sizeof(struct jnixq_struct); break;
		default: return(NULL);
	}
	node = (jnixn)pmalloc(p,sz);
	memset(node,0,sz);
	node->jn.p = p;
	node->jn.node = type;

	va_start(ap,type);
	_jnix_setNode(node,p,ap);
	va_end(ap);

	return(node);
}


void jnix_editNode(jnixn node, ...)
{
	va_list ap;

	va_start(ap,node);
	_jnix_setNode(node,node->jn.p,ap);
	va_end(ap);
}


void _jnix_setNode(jnixn node, pool p, va_list ap)
{
	char *arg;

	while(1) {
		switch(va_arg(ap,int)) {
			/* common */
			case JNIXN_DONE:
				return;
			case JNIXN_ID:
				arg = va_arg(ap,char*);
				if(arg) node->jn.id=pstrdup(p,arg);
				break;
			case JNIXN_FROM:
				arg = va_arg(ap,char*);
				if(arg) node->jn.from=jid_new(p,arg);
				break;
			case JNIXN_TO:
				arg = va_arg(ap,char*);
				if(arg) node->jn.to=jid_new(p,arg);
				break;
			case JNIXN_TYPE:
				node->jn.type = va_arg(ap,int);
				break;
			/* message spesific */
			case JNIXM_BODY:
				arg = va_arg(ap,char*);
				if(arg) node->jm.body=pstrdup(p,arg);
				break;
			case JNIXM_SUBJECT:
				arg = va_arg(ap,char*);
				if(arg) node->jm.subject=pstrdup(p,arg);
				break;
			case JNIXM_THREAD:
				arg = va_arg(ap,char*);
				if(arg) node->jm.thread=pstrdup(p,arg);
				break;
			case JNIXM_PRIORITY:
				node->jm.priority = va_arg(ap,int);
				break;
			/* presence spesific */
			case JNIXP_STATUS:
				arg = va_arg(ap,char*);
				if(arg) node->jp.status=pstrdup(p,arg);
				break;
			case JNIXP_PRIORITY:
				node->jp.priority = va_arg(ap,int);
				break;
			/* iq spesific */
			case JNIXQ_XMLNS:
				arg = va_arg(ap,char*);
				if(arg) node->jq.xmlns=pstrdup(p,arg);
				break;
		}
	}
}


jnixn jnix_parseNode(xmlnode xml)
{
	jnixn node;
	pool p;
	int sz,t;
	char *tmp;
	xmlnode tmpx;

	p = pool_new();

	tmp = xmlnode_get_name(xml);
	if(j_strcmp(tmp,"message")==0) {
		t = JNIXN_MSG;
		sz = sizeof(struct jnixm_struct);
	} else if(j_strcmp(tmp,"presence")==0) {
		t = JNIXN_PRS;
		sz = sizeof(struct jnixp_struct);
	} else if(j_strcmp(tmp,"iq")==0) {
		t = JNIXN_IQ;
		sz = sizeof(struct jnixq_struct);
	} else
		return(NULL);

	node = (jnixn)pmalloc(p,sz);
	memset(node,0,sz);
	node->jn.p = p;
	node->jn.node = t;

	/* common attributes */
	tmp = xmlnode_get_attrib(xml,"id");
	if(tmp) node->jn.id=pstrdup(p,tmp);
	tmp = xmlnode_get_attrib(xml,"from");
	if(tmp) node->jn.from=jid_new(p,tmp);
	tmp = xmlnode_get_attrib(xml,"to");
	if(tmp) node->jn.to=jid_new(p,tmp);
	tmp = xmlnode_get_attrib(xml,"type");
	/* if there isn't a 'type' attribute, uses default type (value 0) */
	if (tmp) {
		int i=0;
		while(_jnix_typelist[i].name!=NULL) {
			if (_jnix_typelist[i].node==t && j_strcmp(_jnix_typelist[i].name,tmp)==0) {
				node->jn.type = _jnix_typelist[i].type;
				break;
			}
			i++;
		}
	}

	switch(t) {
		case JNIXN_MSG:
			/* parse message args */
			tmpx = xmlnode_get_tag(xml,"body");
			if(tmpx) node->jm.body=pstrdup(p,xmlnode_get_data(xmlnode_get_firstchild(tmpx)));
			tmpx = xmlnode_get_tag(xml,"subject");
			if(tmpx) node->jm.subject=pstrdup(p,xmlnode_get_data(xmlnode_get_firstchild(tmpx)));
			tmpx = xmlnode_get_tag(xml,"thread");
			if(tmpx) node->jm.thread=pstrdup(p,xmlnode_get_data(xmlnode_get_firstchild(tmpx)));
			tmpx = xmlnode_get_tag(xml,"priority");
			if(tmpx) node->jm.priority=atoi(xmlnode_get_data(xmlnode_get_firstchild(tmpx)));
			break;
		case JNIXN_PRS:
			/* parse presence args */
			tmpx = xmlnode_get_tag(xml,"status");
			if(tmpx) node->jp.status=pstrdup(p,xmlnode_get_data(xmlnode_get_firstchild(tmpx)));
			tmpx = xmlnode_get_tag(xml,"priority");
			if(tmpx) node->jp.priority=atoi(xmlnode_get_data(xmlnode_get_firstchild(tmpx)));
			break;
		case JNIXN_IQ:
			/* parse iq args */
			tmpx = xmlnode_get_tag(xml,"query");
			tmp = xmlnode_get_attrib(tmpx,"xmlns");
			if(tmp) node->jq.xmlns=pstrdup(p,tmp);
			_jnix_parseItem(p,&node->jq.items,xmlnode_get_firstchild(tmpx));
			break;
	}

	return(node);
}


void jnix_sendNode(jnixc j, jnixn node)
{
	pool p;
	spool xml;
	char *buf;

	if (!j || !node) return;
	if (JNIX_STATE_ONLINE != j->state) return;

	p = pool_new();
	xml = spool_new(p);

	switch(node->jn.node) {
		case JNIXN_MSG: spool_add(xml,"<message"); break;
		case JNIXN_PRS: spool_add(xml,"<presence"); break;
		case JNIXN_IQ: spool_add(xml,"<iq"); break;
	}
	if(node->jn.id) spooler(xml," id='",node->jn.id,"'",xml);
	if(node->jn.from) spooler(xml," from='",jid_full(node->jn.from),"'",xml);
	if(node->jn.to) spooler(xml," to='",jid_full(node->jn.to),"'",xml);
	if(node->jn.type) {
		int i=0;
		while(_jnix_typelist[i].name!=NULL) {
			if (_jnix_typelist[i].node==node->jn.node && _jnix_typelist[i].type==node->jn.type) {
				spooler(xml," type='",_jnix_typelist[i].name,"'",xml);
				break;
			}
			i++;
		}
	}

	switch(node->jn.node) {
		case JNIXN_MSG:
			spool_add(xml,">");
			if(node->jm.thread)
				spooler(xml,"<thread>",strescape(p,node->jm.thread),"</thread>",xml);
			if(node->jm.subject)
				spooler(xml,"<subject>",strescape(p,node->jm.subject),"</subject>",xml);
			if(node->jm.body)
				spooler(xml,"<body>",strescape(p,node->jm.body),"</body>",xml);
			spool_add(xml,"</message>");
			break;
		case JNIXN_PRS:
			if(node->jp.type==JNIXP_TYPE_AVAILABLE || node->jp.type==JNIXP_TYPE_UNAVAILABLE) {
				char buf2[20];
				spool_add(xml,">");
				if(node->jp.status)
					spooler(xml,"<status>",strescape(p,node->jp.status),"</status>",xml);
					sprintf(buf2,"%d",node->jp.priority);
					spooler(xml,"<priority>",buf2,"</priority>",xml);
				spool_add(xml,"</presence>");
			} else {
				spool_add(xml,"/>");
			}
			break;
		case JNIXN_IQ:
			spooler(xml,"><query xmlns='",node->jq.xmlns,"'>",xml);
			spool_add(xml,_jnix_item2str(p,node->jq.items));
			spool_add(xml,"</query></iq>");
			break;
	}

	buf = spool_print(xml);
	xmlstream_write(j,buf);
	pool_free(p);
}


void jnix_freeNode(jnixn node)
{
	pool_free(node->jn.p);
}
