/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */

#include "jnix.h"
#include "xmlstream.h"


jnixr _jnix_parseRoster(xmlnode x)
{
	pool p;
	jnixr rost;
	char *temp, *group;
	xmlnode tempw, tempx, tempy;

	p = pool_new();
	if (!p) return(NULL);
	rost = (jnixr)pmalloc(p,sizeof(_jnixr));
	if (!rost) return(NULL);
	memset(rost,0,sizeof(_jnixr));
	rost->p = p;

	tempw = xmlnode_get_tag(x,"query");
	if (!tempw) return(NULL);

	tempx = xmlnode_get_firstchild(tempw);

	while(tempx) {
		temp = xmlnode_get_name(tempx);
		if (j_strcmp(temp, "group")==0)
			{
			group = xmlnode_get_attrib(tempx,"name");

			if (group)
				{
				tempy = xmlnode_get_firstchild(tempx);
				if (!tempy) _jnix_addRosterItem(rost, JNIXR_TYPE_GROUP, group, NULL, NULL);
				while(tempy) {
					temp = xmlnode_get_name(tempy);
					if (j_strcmp(temp, "item")==0)
						{
						    _jnix_addRosterItem(rost, JNIXR_TYPE_USER, group, xmlnode_get_attrib(tempy,"jid"), xmlnode_get_data(xmlnode_get_firstchild(tempy)));
						}
					tempy = xmlnode_get_nextsibling(tempy);
					}
				}
			}
		tempx = xmlnode_get_nextsibling(tempx);
		}

	return(rost);
}


void _jnix_addRosterItem(jnixr rost, int type, char *group, char *user, char *comment)
{
	jnixritem tsor;

	tsor = pmalloc(rost->p,sizeof(_jnixritem));
	if (!tsor) return;
	memset(tsor,0,sizeof(_jnixritem));

	tsor->type = type;
	if (group) tsor->group = pstrdup(rost->p,group);
	if (user) tsor->user = jid_new(rost->p,user);
	if (comment) tsor->comment = pstrdup(rost->p,comment);

	if (rost->roster) tsor->next = rost->roster;
	rost->roster = tsor;
}


void jnix_getRoster(jnixc c)
{
jnixn node;

node=jnix_newNode(JNIXN_IQ,JNIXN_TYPE,JNIXQ_TYPE_GET,JNIXQ_XMLNS,"jabber:iq:roster",JNIXN_DONE);
jnix_sendNode(c, node);
}


void jnix_getRosterGroup(jnixc c, char *group)
{
jnixn node;

node=jnix_newNode(JNIXN_IQ,JNIXN_TYPE,JNIXQ_TYPE_GET,JNIXQ_XMLNS,"jabber:iq:roster",JNIXN_DONE);
jnix_addItem(node,NULL,"group",NULL,"name",group,NULL);
jnix_sendNode(c, node);
}


void jnix_setRoster(jnixc c, int cmd, char *group, char *user, char *comment)
{
jnixn node;
char *lala;

node=jnix_newNode(JNIXN_IQ,JNIXN_TYPE,JNIXQ_TYPE_SET,JNIXQ_XMLNS,"jabber:iq:roster",JNIXN_DONE);

if (cmd==JNIX_ROSTER_ADD)
	lala = "add";
else if (cmd==JNIX_ROSTER_REMOVE)
	lala = "del";
else
	return;

jnix_addItem(node, NULL, lala, comment, "group", group, NULL);

/* when called with same name, addIQitem adds given attributes */
if (user) jnix_addItem(node, NULL, lala, NULL, "item", user, NULL);

jnix_sendNode(c, node);
}


void jnix_freeRoster(jnixr rost)
{
	pool_free(rost->p);
}
