/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */

#include <stdio.h>
#include <jabber/jnix.h>
#include <ncurses.h>
#include <pthread.h>

#define MAXITEMS 11

jnixc jxc;
WINDOW *mainwin;
char *ujid,*rjid=NULL,*upass,*prompt,connected=0;
int flags=0;

typedef struct {
    char *shortcut;
    char *item;
    void (*func)(char *args);
    char **args;
} menuitem;

void cmd_send(char *args);
void cmd_probe(char *args);
void cmd_status(char *args);
void cmd_getroster(char *args);
void cmd_getrostergroup(char *args);
void cmd_addroster(char *args);
void cmd_register(char *args);
void cmd_help(char *args);
void cmd_quit(char *args);

menuitem menuitems[MAXITEMS] = {
                          { "s", "send", cmd_send, NULL },
                          { "o", "getgroup", cmd_getrostergroup, NULL },
			  { "h", "help", cmd_help, NULL },
                          { "r", "reply", cmd_send, &rjid },
			  { "p", "presence", cmd_status, NULL },
			  { "?", "probe", cmd_probe, NULL },
                          { "w", "roster", cmd_getroster, NULL },
                          { "a", "add", cmd_addroster, NULL },
                          { "g", "register", cmd_register, NULL },
                          { "q", "quit", cmd_quit, NULL },
                          { "q", "exit", cmd_quit, NULL }
                        };


char *pstr[6] = {"available",
		 "unavailable",
		 "subscribe",
		 "unsubscribe",
		 "probe"};

void *jnixpoll(void *arg)
{
 while (1)
  {
  jnix_poll(jxc);
  }
}

void on_auth(jnixc j, jnixa a)
{
 attrset(A_BOLD | COLOR_PAIR(3));
 printw("\n");
 if (a->type==JNIXA_OK)
  printw("Authorization OK");
 else if(a->type==JNIXA_FAIL)
  printw("Authorization FAILED");
 else if(a->type==JNIXA_REGOK)
  printw("Registration OK");
 else if(a->type==JNIXA_REGFAIL)
  printw("Registration FAILED");
 printw(" (%s)\n",jid_full(a->user));
 if(a->msg) printw("reason: %s\n",a->msg);
 attrset(COLOR_PAIR(1));
 printw("%s> ",prompt);
 attrset(COLOR_PAIR(4));
 refresh();
}


void on_connect(jnixc j)
{
 attrset(A_BOLD | COLOR_PAIR(3));
 printw("\nconnected\n");
 attrset(COLOR_PAIR(1));
 printw("%s> ",prompt);
 attrset(COLOR_PAIR(4));
 connected=1;
 refresh();
}

void on_disconnect(jnixc j, int err)
{
 attrset(COLOR_PAIR(2));
 switch(err) {
	case JNIX_OFF_BYE:
		printw("\nServer closed connection. ");
		break;
	case JNIX_OFF_XML:
		printw("\nGot bad xml. ");
		break;
	case JNIX_OFF_SOCK:
		printw("\nNetwork error. ");
		break;
 }
 printw("Disconnected.\n");
 attrset(COLOR_PAIR(1));
 printw("%s> ",prompt);
 attrset(COLOR_PAIR(4));
 connected=0;
 refresh();
 cmd_quit(NULL);
}

void on_message(jnixc j, jnixm msg)
{
 if (rjid) free(rjid);
 rjid = jid_full(msg->from);
 attrset(A_BOLD | COLOR_PAIR(1)); printw("\nfrom: ");
 attrset(COLOR_PAIR(4)); printw("%s\n",rjid);
 attrset(A_BOLD | COLOR_PAIR(1)); printw("subject: ");
 attrset(COLOR_PAIR(4)); printw("%s\n",msg->subject);
 attrset(A_BOLD | COLOR_PAIR(1)); printw("thread: ");
 attrset(COLOR_PAIR(4)); printw("%s\n",msg->thread);
 attrset(A_BOLD | COLOR_PAIR(1)); printw("priority: ");
 attrset(COLOR_PAIR(4)); printw("%d\n",msg->priority);
 attrset(A_BOLD | COLOR_PAIR(1)); printw("message:\n");
 attrset(COLOR_PAIR(4)); printw("%s\n",msg->body);
 attrset(COLOR_PAIR(1));
 printw("%s> ",prompt);
 attrset(COLOR_PAIR(4));
 refresh();
}

void on_presence(jnixc j, jnixp p)
{
 char *tmp;
 tmp = jid_full(p->from);
 attrset(A_BOLD | COLOR_PAIR(5));
 printw("\n%s",tmp);
 attrset(COLOR_PAIR(5));
 printw(" is now ");
 attrset(A_BOLD | COLOR_PAIR(5));
 printw("%s ",pstr[p->type]);
 attrset(COLOR_PAIR(5));
 printw("[%s]\n",p->status);
 attrset(COLOR_PAIR(1));
 printw("%s> ",prompt);
 attrset(COLOR_PAIR(4));
 refresh();
 free(tmp);
}

void on_iqlala(jnixitem item)
{
 jnixitem tmp=item,tmp2;
 while(tmp) {
  printw("tag = '%s'",tmp->element);
  if (tmp->value) printw(", val = '%s'",tmp->value);
  printw("\n");
  tmp2=tmp->attr;
  while(tmp2) {
   printw("attr = '%s'",tmp2->element);
   if (tmp2->value) printw(", val = '%s'",tmp2->value);
   printw("\n");
   tmp2=tmp2->next;
  }
  if(tmp->sub) {
   printw("[\n");
   on_iqlala(tmp->sub);
   printw("]\n");
  }
  tmp = tmp->next;
 }
}

void on_iq(jnixc j, jnixq iq)
{
 attrset(A_BOLD | COLOR_PAIR(5));
 printw("\nGot IQ ");
 switch(iq->type) {
  case JNIXQ_TYPE_SET: printw("set"); break;
  case JNIXQ_TYPE_GET: printw("get"); break;
  case JNIXQ_TYPE_RESULT: printw("result"); break;
 }
 if (iq->from) {
  printw(" from %s",jid_full(iq->from));
 }
 if (iq->xmlns) {
  printw(" namespace %s",iq->xmlns);
 }
 printw("\n");
 on_iqlala(iq->items);
 attrset(COLOR_PAIR(1));
 printw("%s> ",prompt);
 attrset(COLOR_PAIR(4));
 refresh();
}


void on_roster(jnixc j, jnixr r)
{
jnixritem ri;
char *tmp;

if (!r) return;

ri=r->roster;
if (!ri) return;

attrset(A_BOLD | COLOR_PAIR(2));
printw("\n==roster==\n");

while(ri)
	{
	if (ri->type==JNIXR_TYPE_USER)
		{
 		tmp = jid_full(ri->user);
		attrset(COLOR_PAIR(5));
		printw("[%s] ", ri->group);
 		attrset(A_BOLD | COLOR_PAIR(5));
		printw("%40s", tmp);
		attrset(COLOR_PAIR(5));
		printw(" : %s\n", ri->comment);
		}
	else if (ri->type==JNIXR_TYPE_GROUP)
		{
		jnix_getRosterGroup(jxc, ri->group);
		}
	ri=ri->next;
	}

jnix_freeRoster(r);

attrset(COLOR_PAIR(2));
printw("==========\n");

 attrset(COLOR_PAIR(1));
 printw("%s> ",prompt);
 attrset(COLOR_PAIR(4));
 refresh();
}


/* commands */

void cmd_help(char *args)
{
 printw("h help				prints help\n");
 printw("s send <jid>			sends msg\n");
 printw("r reply  			reply\n");
 printw("? probe <jid>                  probes someone's availability\n");
 printw("p presence <online/offline>	set presence\n");
 printw("w roster			get roster (not cached -- realtime!)\n");
 printw("o getgroup <group>		get roster group (not cached -- realtime!)\n");
 printw("a add <jid>			add to roster\n");
 printw("g register			register a new user\n");
 printw("q quit				guess what\n");
 printw("q exit				does the same\n");
 refresh();
}

void cmd_quit(char *args)
{
 attrset(A_BOLD | COLOR_PAIR(1));
 printw("quitting...\n");
 if (connected==1) jnix_disconnect(jxc);
 echo();
 refresh();
 endwin();
 exit(0);
}

void cmd_send(char *args)
{
 char *temp=malloc(256),*subject=malloc(256),*msg=malloc(500);
 jnixn node;

 if (args==NULL) { printw("Usage: send <jid>\n"); refresh(); return; }
 if (connected!=1)
  {
  attrset(A_BOLD | COLOR_PAIR(2));
  printw("not connected\n");
  refresh();
  return;
  }
  
 attrset(A_BOLD | COLOR_PAIR(1)); printw("Subject: "); refresh();
 attrset(COLOR_PAIR(4)); getstr(subject);
 attrset(A_BOLD | COLOR_PAIR(1)); printw("Message:\n");
 refresh();
 strcpy(msg,"");
 for (;;)
  {
  attrset(A_BOLD | COLOR_PAIR(1));
  printw("> ");
  attrset(COLOR_PAIR(4));
  refresh();
  getstr(temp);
  if (strcmp(temp,".")==0) break;
  strcat(msg,temp);
  strcat(msg,"\n");
  }
/* im = jnix_newMessage(args,subject,NULL,0,msg);*/

node = jnix_newNode(JNIXN_MSG,
			JNIXN_TO, args,
			JNIXM_BODY, msg,
			JNIXM_SUBJECT, subject,
			JNIXN_DONE);

 jnix_sendNode(jxc,node);
 jnix_freeNode(node);
 attrset(A_BOLD | COLOR_PAIR(3));
 printw("msg sent\n");

 free(temp);
 free(subject);
 free(msg);
}

void cmd_getrostergroup(char *args)
{
 if (args==NULL) { printw("Usage: getgroup <group>\n"); refresh(); return; }
 if (connected!=1)
  {
  attrset(A_BOLD | COLOR_PAIR(2));
  printw("not connected\n");
  refresh();
  return;
  }
  
 jnix_getRosterGroup(jxc, args);
}

void cmd_register(char *args)
{
 char *user=malloc(256),*pass=malloc(500);

 if (connected!=1)
  {
  attrset(A_BOLD | COLOR_PAIR(2));
  printw("not connected\n");
  refresh();
  return;
  }
  
 attrset(A_BOLD | COLOR_PAIR(1)); printw("Username: "); refresh();
 attrset(COLOR_PAIR(4)); getstr(user);
 attrset(A_BOLD | COLOR_PAIR(1)); printw("Password: "); refresh();
 attrset(COLOR_PAIR(4)); getstr(pass);

 jnix_register(jxc, user, pass);

 attrset(COLOR_PAIR(1));
 printw("%s> ",prompt);
 attrset(COLOR_PAIR(4));
 refresh();

 free(user);
 free(pass);
}

void cmd_probe(char *args)
{
    jnixn node;

    if (!args) { printw("Usage: probe <jid>\n"); refresh(); return; }
    if (connected!=1) {
	attrset(A_BOLD | COLOR_PAIR(2));
	printw("not connected\n");
	refresh();
	return;
    }

    node = jnix_newNode(JNIXN_PRS,
    			JNIXN_TYPE, JNIXP_TYPE_PROBE,
			JNIXN_TO, args,
			JNIXN_DONE);
    jnix_sendNode(jxc,node);
    jnix_freeNode(node);
}

void cmd_status(char *args)
{
 char *pres=NULL,*stat=NULL;
 jnixn node;
 
 if (args==NULL) { printw("Usage: status <online/offline>\n"); refresh(); return; }
 if (connected!=1)
  {
  attrset(A_BOLD | COLOR_PAIR(2));
  printw("not connected\n");
  refresh();
  return;
  }

 if (rindex(strdup(args),' ')!=0)
  {
  pres=strtok(args," ");
  stat=strtok(NULL," ");
//  printw(" -> %s,%s\n",pres,stat);
  refresh();
  }
 else
  {
  pres=args;
//  printw(" -> %s",pres);
  refresh();
  }

 if (strcasecmp(pres,"online")==0)
  {
  node=jnix_newNode(JNIXN_PRS,
  			JNIXN_TYPE, JNIXP_TYPE_AVAILABLE,
                        JNIXP_STATUS, stat,
			JNIXP_PRIORITY, 0,
			JNIXN_DONE);
  jnix_sendNode(jxc,node);
  jnix_freeNode(node);
  attrset(A_BOLD | COLOR_PAIR(3));
  printw("status set\n");
  attrset(COLOR_PAIR(1));
  refresh();
  }
 else if (strcasecmp(pres,"offline")==0)
  {
  node=jnix_newNode(JNIXN_PRS,
  			JNIXN_TYPE,JNIXP_TYPE_UNAVAILABLE,
                        JNIXP_STATUS, stat,
			JNIXP_PRIORITY, 0,
			JNIXN_DONE);
  jnix_sendNode(jxc,node);
  jnix_freeNode(node);
  attrset(A_BOLD | COLOR_PAIR(3));
  printw("status set\n");
  attrset(COLOR_PAIR(1));
  refresh();
  }
 else
  {
  attrset(A_BOLD | COLOR_PAIR(2));
  printw("unknown status\n");
  attrset(COLOR_PAIR(1));
  refresh();
  }
}

void cmd_getroster(char *args)
{
 if (connected!=1)
  {
  attrset(A_BOLD | COLOR_PAIR(2));
  printw("not connected\n");
  refresh();
  return;
  }

jnix_getRoster(jxc);
}


void cmd_addroster(char *args)
{
 char *group=malloc(256),*comment=malloc(500);

 if (args==NULL) { printw("Usage: add <jid>\n"); refresh(); return; }
 if (connected!=1)
  {
  attrset(A_BOLD | COLOR_PAIR(2));
  printw("not connected\n");
  refresh();
  return;
  }
  
 attrset(A_BOLD | COLOR_PAIR(1)); printw("Group: "); refresh();
 attrset(COLOR_PAIR(4)); getstr(group);
 attrset(A_BOLD | COLOR_PAIR(1)); printw("Comment: "); refresh();
 attrset(COLOR_PAIR(4)); getstr(comment);

 jnix_setRoster(jxc, JNIX_ROSTER_ADD, group, args, comment);

 free(group);
 free(comment);
}


int main(int argc, char *argv[])
{
 pthread_t pollth;
 int ch;
 int i,a,b;
 char *temp=malloc(500),*args=NULL;

 if (argc!=3 && argc!=4)
  {
  printf("Usage: [-n] <jid> <pass>\n");
  exit(0);
  }

 prompt = argv[0];
 if (argc==4) {
 flags=JNIXC_NEWUSER;
 ujid = argv[2];
 upass = argv[3];
 } else {
 flags=0;
 ujid = argv[1];
 upass = argv[2];
 }

 mainwin = initscr();
 cbreak();
 echo();
 nonl();
 if (has_colors()) start_color();
 intrflush(stdscr, FALSE);
 scrollok(stdscr, TRUE);
 idlok(stdscr, TRUE);
 keypad(stdscr, TRUE);

 init_pair(1,COLOR_CYAN,COLOR_BLACK);
 init_pair(2,COLOR_RED,COLOR_BLACK);
 init_pair(3,COLOR_GREEN,COLOR_BLACK);
 init_pair(4,COLOR_WHITE,COLOR_BLACK);
 init_pair(5,COLOR_MAGENTA,COLOR_BLACK);

 attrset(A_BOLD | COLOR_PAIR(1));
 printw("connecting...\n");
 refresh();
 jxc = jnix_connect(ujid,upass,flags);
 if (!jxc) {
  printw("cannot connect!\n");
  refresh();
  echo();
  refresh();
  endwin();
  exit(0);
 }
 jnix_setcb(JNIXCB_ONLINE,(jnixEvent)on_connect);
 jnix_setcb(JNIXCB_OFFLINE,(jnixEvent)on_disconnect);
 jnix_setcb(JNIXCB_MESSAGE,(jnixEvent)on_message);
 jnix_setcb(JNIXCB_PRESENCE,(jnixEvent)on_presence);
 jnix_setcb(JNIXCB_IQ,(jnixEvent)on_iq);
 jnix_setcb(JNIXCB_ROSTER,(jnixEvent)on_roster);
 jnix_setcb(JNIXCB_AUTH,(jnixEvent)on_auth);

 if (pthread_create(&pollth,NULL,jnixpoll,NULL)!=0)
   { printf("Error: Unable to create thread\n"); exit(1); }

 while (1)
  {
  attrset(COLOR_PAIR(1));
  printw("%s> ",prompt);
  refresh();
  attrset(COLOR_PAIR(4));
  getstr(temp);
  for (i=0; i<MAXITEMS; i++)
   {
   if (strncmp(menuitems[i].item,temp,strlen(menuitems[i].item))==0 || strncmp(menuitems[i].shortcut,temp,strlen(menuitems[i].shortcut))==0)
    {
    if (rindex(temp,' ')==0)
     {
     if (menuitems[i].args!=NULL)
      {
      menuitems[i].func(*menuitems[i].args);
      }
     else
      {
      menuitems[i].func(NULL);
      }
     }
    else
     {
     strtok(strdup(temp)," ");
     args=strtok(NULL," ");
     refresh();
     menuitems[i].func(args);
     }
    }
   }
  }

 return 0;
}
